@@ -17,14 +17,15 @@ class BluetoothConnection : public esp32_ble_client::BLEClientBase {
   esp_err_t read_characteristic(uint16_t handle);
   esp_err_t write_characteristic(uint16_t handle, const std::string &data, bool response);
   esp_err_t read_descriptor(uint16_t handle);
-  esp_err_t write_descriptor(uint16_t handle, const std::string &data);
+  esp_err_t write_descriptor(uint16_t handle, const std::string &data, bool response);
 
   esp_err_t notify_characteristic(uint16_t handle, bool enable);
 
  protected:
   friend class BluetoothProxy;
+  bool seen_mtu_or_services_{false};
 
-  int16_t send_service_{-1};
+  int16_t send_service_{-2};
   BluetoothProxy *proxy_;
 };
 
