@@ -80,6 +80,10 @@ AURA_LED_COLORS = {
     "RED": AuraLEDColor.RED,
     "BLUE": AuraLEDColor.BLUE,
     "PURPLE": AuraLEDColor.PURPLE,
+    "GREEN": AuraLEDColor.GREEN,
+    "YELLOW": AuraLEDColor.YELLOW,
+    "CYAN": AuraLEDColor.CYAN,
+    "WHITE": AuraLEDColor.WHITE,
 }
 validate_aura_led_colors = cv.enum(AURA_LED_COLORS, upper=True)
 
@@ -132,45 +136,45 @@ CONFIG_SCHEMA = (
 )
 
 
-def to_code(config):
+async def to_code(config):
     var = cg.new_Pvariable(config[CONF_ID])
-    yield cg.register_component(var, config)
+    await cg.register_component(var, config)
     if CONF_PASSWORD in config:
         password = config[CONF_PASSWORD]
         cg.add(var.set_password(password))
-    yield uart.register_uart_device(var, config)
+    await uart.register_uart_device(var, config)
 
     if CONF_NEW_PASSWORD in config:
         new_password = config[CONF_NEW_PASSWORD]
         cg.add(var.set_new_password(new_password))
 
     if CONF_SENSING_PIN in config:
-        sensing_pin = yield cg.gpio_pin_expression(config[CONF_SENSING_PIN])
+        sensing_pin = await cg.gpio_pin_expression(config[CONF_SENSING_PIN])
         cg.add(var.set_sensing_pin(sensing_pin))
 
     for conf in config.get(CONF_ON_FINGER_SCAN_MATCHED, []):
         trigger = cg.new_Pvariable(conf[CONF_TRIGGER_ID], var)
-        yield automation.build_automation(
+        await automation.build_automation(
             trigger, [(cg.uint16, "finger_id"), (cg.uint16, "confidence")], conf
         )
 
     for conf in config.get(CONF_ON_FINGER_SCAN_UNMATCHED, []):
         trigger = cg.new_Pvariable(conf[CONF_TRIGGER_ID], var)
-        yield automation.build_automation(trigger, [], conf)
+        await automation.build_automation(trigger, [], conf)
 
     for conf in config.get(CONF_ON_ENROLLMENT_SCAN, []):
         trigger = cg.new_Pvariable(conf[CONF_TRIGGER_ID], var)
-        yield automation.build_automation(
+        await automation.build_automation(
             trigger, [(cg.uint8, "scan_num"), (cg.uint16, "finger_id")], conf
         )
 
     for conf in config.get(CONF_ON_ENROLLMENT_DONE, []):
         trigger = cg.new_Pvariable(conf[CONF_TRIGGER_ID], var)
-        yield automation.build_automation(trigger, [(cg.uint16, "finger_id")], conf)
+        await automation.build_automation(trigger, [(cg.uint16, "finger_id")], conf)
 
     for conf in config.get(CONF_ON_ENROLLMENT_FAILED, []):
         trigger = cg.new_Pvariable(conf[CONF_TRIGGER_ID], var)
-        yield automation.build_automation(trigger, [(cg.uint16, "finger_id")], conf)
+        await automation.build_automation(trigger, [(cg.uint16, "finger_id")], conf)
 
 
 @automation.register_action(
@@ -185,16 +189,16 @@ def to_code(config):
         key=CONF_FINGER_ID,
     ),
 )
-def fingerprint_grow_enroll_to_code(config, action_id, template_arg, args):
+async def fingerprint_grow_enroll_to_code(config, action_id, template_arg, args):
     var = cg.new_Pvariable(action_id, template_arg)
-    yield cg.register_parented(var, config[CONF_ID])
+    await cg.register_parented(var, config[CONF_ID])
 
-    template_ = yield cg.templatable(config[CONF_FINGER_ID], args, cg.uint16)
+    template_ = await cg.templatable(config[CONF_FINGER_ID], args, cg.uint16)
     cg.add(var.set_finger_id(template_))
     if CONF_NUM_SCANS in config:
-        template_ = yield cg.templatable(config[CONF_NUM_SCANS], args, cg.uint8)
+        template_ = await cg.templatable(config[CONF_NUM_SCANS], args, cg.uint8)
         cg.add(var.set_num_scans(template_))
-    yield var
+    return var
 
 
 @automation.register_action(
@@ -206,10 +210,10 @@ def fingerprint_grow_enroll_to_code(config, action_id, template_arg, args):
         }
     ),
 )
-def fingerprint_grow_cancel_enroll_to_code(config, action_id, template_arg, args):
+async def fingerprint_grow_cancel_enroll_to_code(config, action_id, template_arg, args):
     var = cg.new_Pvariable(action_id, template_arg)
-    yield cg.register_parented(var, config[CONF_ID])
-    yield var
+    await cg.register_parented(var, config[CONF_ID])
+    return var
 
 
 @automation.register_action(
@@ -223,13 +227,13 @@ def fingerprint_grow_cancel_enroll_to_code(config, action_id, template_arg, args
         key=CONF_FINGER_ID,
     ),
 )
-def fingerprint_grow_delete_to_code(config, action_id, template_arg, args):
+async def fingerprint_grow_delete_to_code(config, action_id, template_arg, args):
     var = cg.new_Pvariable(action_id, template_arg)
-    yield cg.register_parented(var, config[CONF_ID])
+    await cg.register_parented(var, config[CONF_ID])
 
-    template_ = yield cg.templatable(config[CONF_FINGER_ID], args, cg.uint16)
+    template_ = await cg.templatable(config[CONF_FINGER_ID], args, cg.uint16)
     cg.add(var.set_finger_id(template_))
-    yield var
+    return var
 
 
 @automation.register_action(
@@ -241,10 +245,10 @@ def fingerprint_grow_delete_to_code(config, action_id, template_arg, args):
         }
     ),
 )
-def fingerprint_grow_delete_all_to_code(config, action_id, template_arg, args):
+async def fingerprint_grow_delete_all_to_code(config, action_id, template_arg, args):
     var = cg.new_Pvariable(action_id, template_arg)
-    yield cg.register_parented(var, config[CONF_ID])
-    yield var
+    await cg.register_parented(var, config[CONF_ID])
+    return var
 
 
 FINGERPRINT_GROW_LED_CONTROL_ACTION_SCHEMA = cv.maybe_simple_value(
@@ -261,13 +265,13 @@ FINGERPRINT_GROW_LED_CONTROL_ACTION_SCHEMA = cv.maybe_simple_value(
     LEDControlAction,
     FINGERPRINT_GROW_LED_CONTROL_ACTION_SCHEMA,
 )
-def fingerprint_grow_led_control_to_code(config, action_id, template_arg, args):
+async def fingerprint_grow_led_control_to_code(config, action_id, template_arg, args):
     var = cg.new_Pvariable(action_id, template_arg)
-    yield cg.register_parented(var, config[CONF_ID])
+    await cg.register_parented(var, config[CONF_ID])
 
-    template_ = yield cg.templatable(config[CONF_STATE], args, cg.bool_)
+    template_ = await cg.templatable(config[CONF_STATE], args, cg.bool_)
     cg.add(var.set_state(template_))
-    yield var
+    return var
 
 
 @automation.register_action(
@@ -283,11 +287,13 @@ def fingerprint_grow_led_control_to_code(config, action_id, template_arg, args):
         }
     ),
 )
-def fingerprint_grow_aura_led_control_to_code(config, action_id, template_arg, args):
+async def fingerprint_grow_aura_led_control_to_code(
+    config, action_id, template_arg, args
+):
     var = cg.new_Pvariable(action_id, template_arg)
-    yield cg.register_parented(var, config[CONF_ID])
+    await cg.register_parented(var, config[CONF_ID])
 
     for key in [CONF_STATE, CONF_SPEED, CONF_COLOR, CONF_COUNT]:
-        template_ = yield cg.templatable(config[key], args, cg.uint8)
+        template_ = await cg.templatable(config[key], args, cg.uint8)
         cg.add(getattr(var, f"set_{key}")(template_))
-    yield var
+    return var
