@@ -1,6 +1,6 @@
 """Constants used by esphome."""
 
-__version__ = "2022.11.2"
+__version__ = "2022.12.2"
 
 ALLOWED_NAME_CHARS = "abcdefghijklmnopqrstuvwxyz0123456789-_"
 
@@ -322,6 +322,7 @@ CONF_INTERNAL = "internal"
 CONF_INTERNAL_FILTER = "internal_filter"
 CONF_INTERNAL_FILTER_MODE = "internal_filter_mode"
 CONF_INTERRUPT = "interrupt"
+CONF_INTERRUPT_PIN = "interrupt_pin"
 CONF_INTERVAL = "interval"
 CONF_INVALID_COOLDOWN = "invalid_cooldown"
 CONF_INVERT = "invert"
@@ -803,6 +804,7 @@ ICON_BRIGHTNESS_6 = "mdi:brightness-6"
 ICON_BUG = "mdi:bug"
 ICON_CHECK_CIRCLE_OUTLINE = "mdi:check-circle-outline"
 ICON_CHEMICAL_WEAPON = "mdi:chemical-weapon"
+ICON_CHIP = "mdi:chip"
 ICON_COUNTER = "mdi:counter"
 ICON_CURRENT_AC = "mdi:current-ac"
 ICON_DATABASE = "mdi:database"
@@ -818,6 +820,7 @@ ICON_GRAIN = "mdi:grain"
 ICON_KEY_PLUS = "mdi:key-plus"
 ICON_LIGHTBULB = "mdi:lightbulb"
 ICON_MAGNET = "mdi:magnet"
+ICON_MEMORY = "mdi:memory"
 ICON_MOLECULE_CO2 = "mdi:molecule-co2"
 ICON_MOTION_SENSOR = "mdi:motion-sensor"
 ICON_NEW_BOX = "mdi:new-box"
@@ -989,6 +992,7 @@ KEY_CORE = "core"
 KEY_TARGET_PLATFORM = "target_platform"
 KEY_TARGET_FRAMEWORK = "target_framework"
 KEY_FRAMEWORK_VERSION = "framework_version"
+KEY_NAME = "name"
 
 # Entity categories
 ENTITY_CATEGORY_NONE = ""
