@@ -1,5 +1,3 @@
-# pylint: disable=wrong-import-position
-
 import base64
 import codecs
 import collections
@@ -10,11 +8,12 @@ import json
 import logging
 import multiprocessing
 import os
-from pathlib import Path
 import secrets
 import shutil
 import subprocess
 import threading
+from pathlib import Path
+from typing import Optional
 
 import tornado
 import tornado.concurrent
@@ -22,15 +21,15 @@ import tornado.gen
 import tornado.httpserver
 import tornado.ioloop
 import tornado.iostream
-from tornado.log import access_log
 import tornado.netutil
 import tornado.process
 import tornado.web
 import tornado.websocket
+from tornado.log import access_log
 
 from esphome import const, platformio_api, util, yaml_util
 from esphome.core import EsphomeError
-from esphome.helpers import mkdir_p, get_bool_env, run_system_command
+from esphome.helpers import get_bool_env, mkdir_p, run_system_command
 from esphome.storage_json import (
     EsphomeStorageJSON,
     StorageJSON,
@@ -38,14 +37,11 @@ from esphome.storage_json import (
     ext_storage_path,
     trash_storage_path,
 )
-from esphome.util import shlex_quote, get_serial_ports
-from .util import password_hash
-
-# pylint: disable=unused-import, wrong-import-order
-from typing import Optional  # noqa
-
+from esphome.util import get_serial_ports, shlex_quote
 from esphome.zeroconf import DashboardImportDiscovery, DashboardStatus, EsphomeZeroconf
 
+from .util import password_hash
+
 _LOGGER = logging.getLogger(__name__)
 
 ENV_DEV = "ESPHOME_DASHBOARD_DEV"
@@ -190,7 +186,6 @@ def websocket_method(name):
     return wrap
 
 
-# pylint: disable=abstract-method, arguments-differ
 @websocket_class
 class EsphomeCommandWebSocket(tornado.websocket.WebSocketHandler):
     def __init__(self, application, request, **kwargs):
@@ -421,6 +416,10 @@ class ImportRequestHandler(BaseHandler):
             self.set_status(500)
             self.write("File already exists")
             return
+        except ValueError:
+            self.set_status(422)
+            self.write("Invalid package url")
+            return
 
         self.set_status(200)
         self.finish()
@@ -694,6 +693,24 @@ class PrometheusServiceDiscoveryHandler(BaseHandler):
         self.write(json.dumps(sd))
 
 
+class BoardsRequestHandler(BaseHandler):
+    @authenticated
+    def get(self):
+        from esphome.components.esp32.boards import BOARDS as ESP32_BOARDS
+        from esphome.components.esp8266.boards import BOARDS as ESP8266_BOARDS
+        from esphome.components.rp2040.boards import BOARDS as RP2040_BOARDS
+
+        boards = {
+            "esp32": {key: val[const.KEY_NAME] for key, val in ESP32_BOARDS.items()},
+            "esp8266": {
+                key: val[const.KEY_NAME] for key, val in ESP8266_BOARDS.items()
+            },
+            "rp2040": {key: val[const.KEY_NAME] for key, val in RP2040_BOARDS.items()},
+        }
+        self.set_header("content-type", "application/json")
+        self.write(json.dumps(boards))
+
+
 class MDNSStatusThread(threading.Thread):
     def run(self):
         global IMPORT_RESULT
@@ -797,7 +814,6 @@ class EditRequestHandler(BaseHandler):
         filename = settings.rel_path(configuration)
         content = ""
         if os.path.isfile(filename):
-            # pylint: disable=no-value-for-parameter
             with open(file=filename, encoding="utf-8") as f:
                 content = f.read()
         self.write(content)
@@ -805,7 +821,6 @@ class EditRequestHandler(BaseHandler):
     @authenticated
     @bind_config
     def post(self, configuration=None):
-        # pylint: disable=no-value-for-parameter
         with open(file=settings.rel_path(configuration), mode="wb") as f:
             f.write(self.request.body)
         self.set_status(200)
@@ -849,7 +864,7 @@ PING_REQUEST = threading.Event()
 class LoginHandler(BaseHandler):
     def get(self):
         if is_authenticated(self):
-            self.redirect("/")
+            self.redirect("./")
         else:
             self.render_login_page()
 
@@ -894,7 +909,7 @@ class LoginHandler(BaseHandler):
         password = self.get_argument("password", "")
         if settings.check_password(username, password):
             self.set_secure_cookie("authenticated", cookie_authenticated_yes)
-            self.redirect("/")
+            self.redirect("./")
             return
         error_str = (
             "Invalid username or password" if settings.username else "Invalid password"
@@ -1066,6 +1081,7 @@ def make_app(debug=get_bool_env(ENV_DEV)):
             (f"{rel}json-config", JsonConfigRequestHandler),
             (f"{rel}rename", EsphomeRenameHandler),
             (f"{rel}prometheus-sd", PrometheusServiceDiscoveryHandler),
+            (f"{rel}boards", BoardsRequestHandler),
         ],
         **app_settings,
     )
